@@ -2,7 +2,7 @@
 
 # MIT License
 #
-# Copyright 2018-2020 New York University Abu Dhabi
+# Copyright 2018-2021 New York University Abu Dhabi
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -281,6 +281,7 @@ class Analyzer:
                     merged['stem'] = stem_feats['diac']
                     merged['stemcat'] = stem_cat
                     merged['source'] = 'backoff'
+                    merged['pattern'] = 'backoff'
                     merged['gloss'] = stem_feats['gloss']
 
                     combined.append(merged)
@@ -316,7 +317,7 @@ class Analyzer:
             result['stem'] = word
             result['stemgloss'] = word
             result['stemcat'] = None
-            result['lex'] = word + '_0'
+            result['lex'] = word
             result['bw'] = word + '/NOUN_NUM'
             result['source'] = 'digit'
 
@@ -337,6 +338,12 @@ class Analyzer:
             result['lex_logprob'] = -99.0
             result['pos_lex_logprob'] = -99.0
 
+            if 'form_gen' in self._db.defines and result['gen'] == '-':
+                result['gen'] = result['form_gen']
+
+            if 'form_num' in self._db.defines and result['num'] == '-':
+                result['num'] = result['form_num']
+
             return [result]
 
         elif _is_punc(word):
@@ -345,7 +352,7 @@ class Analyzer:
             result['stem'] = word
             result['stemgloss'] = word
             result['stemcat'] = None
-            result['lex'] = word + '_0'
+            result['lex'] = word
             result['bw'] = word + '/PUNC'
             result['source'] = 'punc'
 
@@ -366,6 +373,12 @@ class Analyzer:
             result['lex_logprob'] = -99.0
             result['pos_lex_logprob'] = -99.0
 
+            if 'form_gen' in self._db.defines and result['gen'] == '-':
+                result['gen'] = result['form_gen']
+
+            if 'form_num' in self._db.defines and result['num'] == '-':
+                result['num'] = result['form_num']
+
             return [result]
 
         elif _has_punc(word):
@@ -377,7 +390,7 @@ class Analyzer:
             result['stem'] = word
             result['stemgloss'] = word
             result['stemcat'] = None
-            result['lex'] = word + '_0'
+            result['lex'] = word
             result['bw'] = word + '/FOREIGN'
             result['source'] = 'foreign'
 
@@ -399,6 +412,12 @@ class Analyzer:
             result['lex_logprob'] = -99.0
             result['pos_lex_logprob'] = -99.0
 
+            if 'form_gen' in self._db.defines and result['gen'] == '-':
+                result['gen'] = result['form_gen']
+
+            if 'form_num' in self._db.defines and result['num'] == '-':
+                result['num'] = result['form_num']
+
             return [result]
 
         else:
@@ -471,3 +490,24 @@ class Analyzer:
         '''
 
         return list(map(lambda w: AnalyzedWord(w, self.analyze(w)), words))
+
+    def all_feats(self):
+        """Return a set of all features provided by the database used in this
+        analyzer instance.
+
+        Returns:
+            :obj:`frozenset` of :obj:`str`: The set all features provided by
+            the database used in this analyzer instance.
+        """
+
+        return self._db.all_feats()
+
+    def tok_feats(self):
+        """Return a set of tokenization features provided by the database used
+        in this analyzer instance.
+
+        Returns:
+            :obj:`frozenset` of :obj:`str`: The set tokenization features
+            provided by the database used in this analyzer instance.
+        """
+        return self._db.tok_feats()
